import System.Text.RegularExpressions
import AlbumArtDownloader.Scripts
import util

class LastFmArtist(AlbumArtDownloader.Scripts.IScript, ICategorised):
	Name as string:
		get: return "LastFM Artist"
	Version as string:
		get: return "0.8"
	Author as string:
		get: return "Alex Vallat, pochaboo"
	Category as string:
		get: return "Artist Images"	
	protected regexStr as string:
		get: return "<a href=\"(?<url>)[^\"]+\" class=\"image-list-item\">\\s*<img[^>]*?src=\"(?<imageUrlBase>[^\"]+?/u/)[^/]+/(?<id>[^\"]+)\"\\s+alt=\"(?<title>[^\"]+)"
	protected requestUrlBase as string:
		get: return "https://www.last.fm"
	
	def AddResults(imageMatches as MatchCollection, results as IScriptResults):
		results.EstimatedCount += imageMatches.Count
		for imageMatch as Match in imageMatches:
			imageUrl = imageMatch.Groups["imageUrlBase"].Value + imageMatch.Groups["id"].Value
			imageUrlThumbnail = imageMatch.Groups["imageUrlBase"].Value +  "60x60/" + imageMatch.Groups["id"].Value
			results.Add(imageUrlThumbnail, imageMatch.Groups["title"].Value, imageMatch.Groups["url"].Value, -1, -1, imageUrl)
		
	def Search(artist as string, album as string, results as IScriptResults):
		artist = StripCharacters("&.'\";:?!", artist)
		
		encodedArtist = EncodeUrl(artist)
		requestUrl = "${requestUrlBase}/music/${encodedArtist}/+images"	
		
		imagesHtml = GetPage(requestUrl)
		imageMatches = Regex(regexStr, RegexOptions.Singleline).Matches(imagesHtml)
		AddResults(imageMatches, results)
		
		pageCount = 1
		try:
			pageCountMatch = Regex("<a href=\"\\?page=(?<pageCount>\\d+)\"(?:(?!<li).)*<li class=\"pagination-next\">", RegexOptions.Singleline).Match(imagesHtml)
			pageCount = System.Int32.Parse(pageCountMatch.Groups["pageCount"].Value)
		except:
			return
			
		for i in range(2, pageCount + 1):
			requestUrl = "${requestUrlBase}/music/${encodedArtist}/+images" + "?page=" + i.ToString("D")
			imagesHtml = GetPage(requestUrl)
			imageMatches = Regex(regexStr, RegexOptions.Singleline).Matches(imagesHtml)
			AddResults(imageMatches, results)
		
	def RetrieveFullSizeImage(fullSizeCallbackParameter):
		return fullSizeCallbackParameter;